/***************************************************************************

FILE: 65c816.cpp

PURPOSE: This is the main CPU emulation engine.

Copyright 1997 Chad Kitching.  Read the readme.txt for license info.

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <io.h>
#include <dos.h>
#include <malloc.h>
#include <mem.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <time.h>
#include "snesstru.h"
#include "memory~1.h"
#include "65c816.h"
#include "regist~1.h"
#include "opcodes1.h"

unsigned short int PC,X,Y,A,D,S;
unsigned char DB,PB;
bool TrippedNMI;
ProcessorFlags P;
int t,totalt;

void (*Opcode[258])();        //I know that there are only 255 opcodes, but less than this, and it overwrites CartROM...

void InitEngine()
{
  int i,j;

  for (i=0;i<=255;i++) Opcode[i]=debug;

  /*Opcode[0x9c]=stz_a;
  Opcode[0xD8]=cld;
  Opcode[0x18]=clc;
  Opcode[0xFB]=xce;
  Opcode[0xAA]=tax;
  Opcode[0x8A]=txa;
  Opcode[0xA9]=lda_i;
  Opcode[0xC2]=rep;
  Opcode[0xE2]=sep;
  Opcode[0xA2]=ldx_i;
  Opcode[0x5B]=tcd;
  Opcode[0x1B]=tcs;
  Opcode[0x48]=pha;
  Opcode[0xAB]=plb;
  Opcode[0x9A]=txs;
  Opcode[0x20]=jsr_a;
  Opcode[0x58]=cli;
  Opcode[0x4B]=phk;
  Opcode[0x8D]=sta_a;
  Opcode[0xBF]=lda_al_x;
  Opcode[0x9F]=sta_al_x;
  Opcode[0xCA]=dex;
  Opcode[0xEA]=nop;
  Opcode[0xE8]=inx;
  Opcode[0x22]=jsl_al;
  Opcode[0xA4]=ldy_d;
  Opcode[0x8E]=stx_a;
  Opcode[0xBC]=ldy_a_x;
  Opcode[0xE0]=cpx_i;
  Opcode[0x86]=stx_d;
  Opcode[0x60]=rts;
  Opcode[0x08]=php;
  Opcode[0xA0]=ldy_i;
  Opcode[0xAF]=lda_al;
  Opcode[0x5C]=jmp_al;
  Opcode[0xD0]=bne_r;
  Opcode[0xDB]=stp;
  Opcode[0xEB]=xba;
  Opcode[0x8C]=sty_a;
  Opcode[0xAD]=lda_a;
  Opcode[0x42]=wdm;
  Opcode[0x29]=and_i;
  Opcode[0xF0]=beq_r;
  Opcode[0xBD]=lda_a_x;
  Opcode[0x40]=rti;
  Opcode[0xC0]=cpy_i;
  Opcode[0x28]=plp;
  Opcode[0x90]=bcc_r;*/
  Opcode[0x21]=Opcode[0x23]=Opcode[0x25]=Opcode[0x27]=Opcode[0x29]=Opcode[0x2D]=Opcode[0x2F]=_AND_;
  Opcode[0x31]=Opcode[0x32]=Opcode[0x33]=Opcode[0x35]=Opcode[0x37]=Opcode[0x39]=Opcode[0x3D]=Opcode[0x3F]=_AND_;
  Opcode[0x78]=sei;
  Opcode[0x4B]=phk;
  Opcode[0xAB]=plb;
  Opcode[0x18]=clc;
  Opcode[0xFB]=xce;
  Opcode[0xE2]=sep;
  Opcode[0xC2]=rep;
  Opcode[0xA1]=Opcode[0xA3]=Opcode[0xA5]=Opcode[0xA7]=Opcode[0xA9]=Opcode[0xAD]=Opcode[0xAF]=_LDA_;
  Opcode[0xB1]=Opcode[0xB2]=Opcode[0xB3]=Opcode[0xB5]=Opcode[0xB7]=Opcode[0xB9]=Opcode[0xBD]=Opcode[0xBF]=_LDA_;
  Opcode[0x81]=Opcode[0x83]=Opcode[0x85]=Opcode[0x87]=Opcode[0x89]=Opcode[0x8D]=Opcode[0x8F]=_STA_;
  Opcode[0x91]=Opcode[0x92]=Opcode[0x93]=Opcode[0x95]=Opcode[0x97]=Opcode[0x99]=Opcode[0x9D]=Opcode[0x9F]=_STA_;
  Opcode[0xA2]=Opcode[0xA6]=Opcode[0xAE]=Opcode[0xB6]=Opcode[0xBE]=_LDX_;
  Opcode[0xE8]=inx;
  Opcode[0xEA]=nop;
  Opcode[0xA0]=Opcode[0xA4]=Opcode[0xAC]=Opcode[0xB4]=Opcode[0xBC]=_LDY_;
  Opcode[0xE0]=Opcode[0xE4]=Opcode[0xEC]=_CPX_;
  Opcode[0xD0]=bne;
  Opcode[0x86]=Opcode[0x8E]=Opcode[0x96]=Opcode[0x9E]=_STX_;
  Opcode[0x9C]=stz_a;
  Opcode[0x9E]=stz_ax;
  Opcode[0x8A]=txa;
  Opcode[0xEB]=xba;
  Opcode[0x20]=jsr_a;
  Opcode[0x08]=php;
  Opcode[0x28]=plp;
  Opcode[0x84]=Opcode[0x8C]=Opcode[0x94]=_STY_;
  Opcode[0xF0]=beq;
  Opcode[0x60]=rts;
  Opcode[0xCA]=dex;
  Opcode[0x80]=bra;
  Opcode[0xC0]=Opcode[0xC4]=Opcode[0xCC]=_CPY_;
  Opcode[0xC8]=iny;
  Opcode[0x58]=cli;
  Opcode[0x98]=tya;
  Opcode[0x61]=Opcode[0x63]=Opcode[0x65]=Opcode[0x67]=Opcode[0x69]=Opcode[0x6D]=Opcode[0x6F]=_ADC_;
  Opcode[0x71]=Opcode[0x72]=Opcode[0x73]=Opcode[0x75]=Opcode[0x77]=Opcode[0x79]=Opcode[0x7D]=Opcode[0x7F]=_ADC_;
  Opcode[0xA8]=tay;
  Opcode[0x48]=pha;
  Opcode[0x5C]=jmp_al;

  j=0;
  for (i=0;i<=255;i++) if (Opcode[i]!=debug) j++;
  P.M=1;
  P.X=1;
  P.D=0;
  P.I=1;
  P.C=P.C=1;
  D=X=Y=A=0;
  PB=DB=0;
  S=0x0100;

  printf("There are %i implemented opcodes, %i remain unimplmented.\n",j,256-j);
  printf("There are %i implemented addressing modes out of 24.\n",12);
}

void BeginEmulation ()
{
  unsigned int NMI, Reset;
  Test1();
  puts("EMULATION BEGUN!!!");
  puts("Init Screen - Nope");
  Reset=(int)SNESgetword(0xfffc,0);
  if (Reset<0x7fffl)
  {
    printf("File header is corrupted.  Cannot start game in RAM, defaulting to reset at 0x8000");
    Reset=0x8000;
  }
  PC=Reset;
  NMI=SNESgetword(0xfffa,0);
  clock();
  //printf ("%X, %X %X\n",PC,(int)SNESgetbyte(0xfffc,0),(int)SNESgetbyte(0xfffd,0));
  do
  {
   #ifdef debug1
   printf ("%x:%x %x ",PB,PC, SNESgetbyte(PC,PB));
   #endif
   Opcode[SNESgetbyte(PC,PB)]();
   #ifdef debug1
   fflush(stdout);
   #endif
   if (t>=53600)
     {
      #ifdef debug1
      puts("NMI Tripped!!!");
      #endif
      //exit(9);
      totalt+=t;
      if ((unsigned short int)NMI>0x8000)
        DoNMI();
      else
        t=0;
      TrippedNMI=true;
     }
   //PC++;
   //if (PC==0x8187) debug();
  } while (!kbhit());
  debug();

}

void Test1()
{
  //int i;

  //for (i=0xFFC0;i<0xFFFF;i++) printf("%x %c ",i,SNESgetbyte(i,0));

  printf("NMI Vector: 0x%X\nReset Vector: 0x%X\n",SNESgetword(0xfffa,0),SNESgetword(0xfffc,0));
}
