/***************************************************************************

FILE: opcodes1.cpp

PURPOSE: This encompasses the CPU opcodes.

Copyright 1997 Chad Kitching.  Read the readme.txt for license info.

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <io.h>
#include <dos.h>
#include <malloc.h>
#include <mem.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <time.h>
#include "snesstru.h"
#include "memory~1.h"
#include "65c816.h"
#include "opcodes1.h"
#include "regist~1.h"
#include "adrsmode.h"

//The annoying message you will see with nearly every rom you try.

void debug()
{
 int stoptime=clock();
  printf("???\nUnimplemented Opcode encountered (0x%x) at %x:%x\n",SNESgetbyte(PC,PB),PB,PC);
  printf("  Flags:  N:%i V:%i M:%i X:%i D:%i I:%i Z:%i C:%i E:%i\n",P.N,P.V,P.M,P.X,P.D,P.I,P.Z,P.C,P.E);
  printf("  Regs:   A: $%04x D: $%04x X: $%04x Y: $%04x\n",A,D,X,Y);
  printf("  Regs:   PB:  $%02x PC:$%04x DB:  $%02x S: $%04x\n",PB,PC,DB,S);
  printf("  Machine cycles elapsed: %i\n",t+totalt);
  if (stoptime!=0)
    printf("  Clock cycles elapsed:  %i\n  Speed (Instr/S): %i\n",stoptime,(t+totalt)/stoptime*CLOCKS_PER_SEC);
  else
    printf("  Clock cycles elapsed:  0\n  Approx speed (instr/s): >%i\n",(t+totalt)*CLOCKS_PER_SEC);
  exit(4);
}

//These are my stack routines

void pushb(char data)
{
 int location;
 MemType typ;
#ifdef debug1
 printf("Pushed Byte: %x\n",data);
#endif
 SnesMap(S, 0, &typ, (long int*)&location);
 SNESsetbyte(typ,location,data);
 S++;
}

void pushw(short int data)
{
 int location;
 MemType typ;
#ifdef debug1
 printf("Pushed Word: %x\n",data);
#endif
 SnesMap(S, 0, &typ, (long int*)&location);
 SNESsetword(typ,location,data);
 S+=2;
}

char pullb()
{
 char tmp;

 S--;
 tmp=SNESgetbyte(S,0);
#ifdef debug1
 printf("Pulled Byte:");
 printf("%x\n",tmp);
#endif
 return tmp;
}

short int pullw()
{
 int tmp;
 S-=2;
 tmp=SNESgetword(S,0);
#ifdef debug1
 printf("Pulled Word:");
 printf("%x\n",tmp);
#endif
 return tmp;
}

//older shit (obsolete)

void lda_i()
{
  printf("lda #   ");
  PC++;
  if (P.M==1)
    {
     A=SNESgetbyte(PC,PB);
    }
   else
    {
     A=SNESgetword(PC,PB);
     PC++;
    }
  t+=2;
  printf("A=%x\n",A);
}

void ldx_i()
{
  printf("ldx #   ");
  PC++;
  if (P.X==1)
    {
     X=SNESgetbyte(PC,PB);
    }
   else
    {
     X=SNESgetword(PC,PB);
     PC++;
    }
  t+=2;
  printf("X=%x\n",X);
}

void tcd()
{
 printf("tcd \n");
 D=A;
 t+=2;
}

void tcs()
{
 printf("tcs\n");
 S=A;
 t+=2;
}

void txs()
{
 printf("txs\n");
 if (P.E==1)
   S=X&0xFF|0x100;
 else
   S=X;
 t+=2;
}

void sta_a()
{
 int location;
 MemType typ;

 printf("sta a\n");
 PC++;
 SnesMap(SNESgetword(PC,PB),DB, &typ, (long int*)&location);
 if (P.M==1)
   SNESsetbyte(typ,location,(char)(A&0xff));
  else
   SNESsetword(typ,location,A);
 PC++;
 t+=4;
}

void lda_al_x()          //how does this opcode work in EMULATION mode? (P.E=1)
{
 short int tmp;
 unsigned char tmp2;
 printf("lda al,x      ");
 PC++;
 tmp2=SNESgetbyte(PC,PB);
 PC++;
 tmp=SNESgetword(PC,PB);
 if ((long int)(tmp+X)>65535)
   {
    tmp2++;
    tmp+=X;
   }
 else
   tmp+=X;
 if (P.M==1)
   A=SNESgetbyte(tmp,tmp2);
 else
   A=SNESgetword(tmp,tmp2);
 PC++;
 t+=5;
 printf("A=%x\n",A);
}             //That was a pain...  But the next will be worse...  Ugg.

void sta_al_x()          //how does this opcode work in EMULATION mode? (P.E=1)
{
 short int tmp;
 unsigned char tmp2;
 int location;
 MemType typ;

 printf("sta al,x\n");
 PC++;
 tmp2=SNESgetbyte(PC,PB);
 PC++;
 tmp=SNESgetword(PC,PB);
 if ((long int)(tmp+X)>65535)
   {
    tmp2++;
    tmp+=X;
   }
 else
   tmp+=X;

 if (P.M==1)
   {
     SnesMap(tmp,tmp2, &typ, (long int*)&location);
     SNESsetbyte(typ,location,A);
   }
 else
   {
     SnesMap(tmp,tmp2, &typ, (long int*)&location);
     SNESsetword(typ,location,A);
   }
 PC++;
 t+=5;
}

void dex()        //heh  Now a simple opcode....  I've tortured myself enough
{
 #ifdef debug1
 printf("dex\n");
 #endif
 X--;
 if (P.X==1)
   X&=0xFF;
 t+=2;
 PC++;
}

void jsl_al()
{
 short int oldpc;

 printf("jsl al\n");
 pushb(PB);
 pushw(PC+4);
 PC++;
 oldpc=PC;
 PC=SNESgetword(PC,PB);
 PC--;
 PB=SNESgetword(oldpc+2,PB);
 t+=8;
}

void ldy_d()
{
 printf("ldy d     ");
 PC++;
 if (P.X==1)
   Y=SNESgetbyte(D+SNESgetbyte(PC,PB),DB);
 else
   Y=SNESgetword(D+SNESgetbyte(PC,PB),DB);
 t+=3;
 printf("Y=%x\n",Y);
}

void stx_a()
{
 int location;
 MemType typ;

 printf("stx a\n");
 PC++;
 SnesMap(SNESgetword(PC,PB),DB, &typ, (long int*)&location);
 if (P.X==1)
   SNESsetbyte(typ,location,(char)(X&0xff));
  else
   SNESsetword(typ,location,X);
 PC++;
 t+=4;
}

void ldy_a_x()
{
 short int tmpaddr;
 printf("ldy a,x      ");
 PC++;
 if (P.X==1)
   {
    tmpaddr=SNESgetword(PC,PB)+(X&0xFF);
    Y=SNESgetbyte(tmpaddr,DB);
   }
 else
   {
    tmpaddr=SNESgetword(PC,PB)+X;
    Y=SNESgetword(tmpaddr,DB);
   }
 PC++;
 t+=4;
 printf("Y=%x\n",Y);
}

void cpx_i()
{
 printf("cpx #$");
 PC++;
 if (P.X==1)
   {
    if ((unsigned char)SNESgetbyte(PC,PB)==(unsigned char)(X&0xFF))
      P.Z=0;
    else
      P.Z=1;
    printf("%x      X==%x  P.Z==%i\n",SNESgetbyte(PC,PB),X,P.Z);
   }
 else
   {
    if ((unsigned short int)SNESgetword(PC,PB)==(unsigned short int)X)
      P.Z=0;
    else
      P.Z=1;
    PC++;
    printf("%x      X==%x  P.Z==%i\n",SNESgetword(PC,PB),X,P.Z);
   }
 t+=3;
}

void stx_d()
{
 int location;
 MemType typ;

 printf("stx d\n");
 PC++;
 SnesMap(SNESgetword(PC,PB)+D,DB, &typ, (long int*)&location);
 if (P.X==1)
   SNESsetbyte(typ,location,(char)(X&0xff));
  else
   SNESsetword(typ,location,X);
 t+=3;
}

void ldy_i()
{
 printf("ldy #      ");
 PC++;
 if (P.X==1)
   Y=SNESgetbyte(PC,PB);
 else
   Y=SNESgetword(PC++,PB);
 t+=2;
 printf("Y=%x\n",Y);
}

void lda_al()
{
 printf("lda al     ");
 PC++;
 if (P.M==1)
   A=SNESgetbyte(SNESgetword(PC++,PB),SNESgetbyte(PC,PB));
  else
   A=SNESgetword(SNESgetword(PC++,PB),SNESgetbyte(PC,PB));
 PC++;
 t+=5;
 printf("A=%x\n",A);
}

void jmp_al()
{
 unsigned short int tmp;

 #ifdef debug1
 printf("jmp al\n");
 #endif
 PC++;
 tmp=SNESgetword(PC,PB);
 PB=SNESgetbyte(PC+2,PB);
 PC=tmp;
 t+=4;
}

void stp()
{
 printf("stp/hlt\n");
 t+=3;
}

void sty_a()
{
 int location;
 MemType typ;

 printf("sty a\n");
 PC++;
 SnesMap(SNESgetword(PC,PB)+D,DB, &typ, (long int*)&location);
 if (P.M==1)
   SNESsetbyte(typ,location,A&0xff);
 else
   SNESsetword(typ,location,A);
 PC++;
 t+=4;
}

void lda_a()
{
 printf("lda $%x     ",SNESgetword(PC+1,PB));
 PC++;
 if (P.M==1)
   A=SNESgetbyte(SNESgetword(PC,PB),PB);
 else
   A=SNESgetword(SNESgetword(PC,PB),PB);
 PC++;
 t+=4;
 printf("A=%x\n",A);
}

void wdm()
{
 printf("wdm  --  Probably error in emulation engine...\n");
 debug();              //for now, until the engine works...
 t+=2;
 PC++;
}

void and_i()
{
 printf("and #%x    ",SNESgetbyte(PC+1,PB));
 PC++;
 printf("A before=%x ",A);
 if (P.M==1)
   A=A&SNESgetbyte(PC,PB);
 else
   {
    A=A&SNESgetword(PC,PB);
    PC++;
   }
 if ((unsigned int)A>0)
   P.Z=0;
 else
   P.Z=1;
 t+=2;
 printf("A after=%x \n",A);
}

void beq_r()
{
 printf("beq $%x\n",(signed char)SNESgetbyte(PC+1,PB));
 PC++;
 if (P.Z==1) PC+=(signed char)SNESgetbyte(PC,PB);       //Hmmm.  Didn't need to decrement for negative....  Suspicious.
 t+=2;
}

void lda_a_x()
{
 short int tmpaddr;
 printf("lda $%x,X      ",SNESgetword(PC+1,PB));
 PC++;
 if (P.X==1)
   {
    tmpaddr=SNESgetword(PC,PB)+(X&0xFF);
    A=SNESgetbyte(tmpaddr,DB);
   }
 else
   {
    tmpaddr=SNESgetword(PC,PB)+X;
    A=SNESgetword(tmpaddr,DB);
   }
 PC++;
 t+=4;
 printf("A=%x\n",Y);
}

void and_id_s_y()
{
 unsigned short int tmpaddr;
 printf("and (d,s),y\n");
 PC++;
 tmpaddr=SNESgetbyte(PC,PB)+S;
 tmpaddr=SNESgetword(tmpaddr,0)+Y;
 if (P.M==1)
   A=A&SNESgetbyte(tmpaddr,DB);
 else
   A=A&SNESgetword(tmpaddr,DB);
 if (A>0)
   P.Z=0;
 else
   P.Z=1;
 t+=7;
}

void rti()
{
 plp();
 DB=pullb();
 D=pullw();
 Y=pullw();
 X=pullw();
 A=pullw();
 PC=pullw();
 PB=pullb();
 t+=7;
}

void cpy_i()
{
 printf("cpy #\n");
 PC++;
 if (P.X==1)
   {
    if (SNESgetbyte(PC,PB)!=(Y&0xFF))
      P.Z=1;
    else
      P.Z=0;
   }
 else
   {
    if (SNESgetword(PC,PB)!=Y)
      P.Z=1;
    else
      P.Z=0;
    PC++;
   }
 t+=3;
}

void bcc_r()
{
 printf("bcc $%x\n",(signed char)SNESgetbyte(PC+1,PB));
 PC++;
 if (P.C==1) PC+=(signed char)SNESgetbyte(PC,PB);       //Hmmm.  Didn't need to decrement for negative....  Suspicious.
 t+=2;
}

//special functions

void nil()
{}

void DoNMI()       //Not really an opcode, but I need access to stack routines.
{
 pushb(PB);
 pushw(PC);
 pushw(A);
 pushw(X);
 pushw(Y);
 pushw(D);
 pushb(DB);
 php();
 PC=SNESgetword(0xFFFA,0);        //This is somewhat odd.  Sometimes == 0?
 PB=0;
 t=0;
}

//thess are the addressing mode families.

inline unsigned char family1b()
{
 unsigned char optest;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 1:      //direct indexed indirect
        return direct_indexed_indirVb();
   case 3:      //stack relative
        debug();
        break;
   case 5:      //direct
        return directVb();
   case 7:      //direct indirect long
        return direct_indir_longVb();
   case 9:      //immediate
        return SNESgetbyte(PC++,PB);
   case 0xd:    //absolute
        return absoluteVb();
   case 0xf:    //absolute
        return absolute_longVb();
   case 0x11:   //direct Indirect indexed
        return direct_indir_indexed_yVb();
   case 0x12:   //direct indirect
        return direct_indirVb();
   case 0x13:   //stack relative indirect indexed
        debug();
        break;
   case 0x15:   //direct indexed x
        return direct_indexed_xVb();
   case 0x17:   //direct indirect indexed long
        debug();
        break;
   case 0x19:   //absolute indexed (y)
        return absolute_YindexedVb();
   case 0x1D:
        return absolute_XindexedVb();
   case 0x1F:   //absolute indexed long
        return absolute_long_indexedVb();
   }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

inline unsigned char family1w()
{
 unsigned char optest;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 1:      //direct indexed indirect
        return direct_indexed_indirVw();
   case 3:      //stack relative
        debug();
        break;
   case 5:      //direct
        return directVw();
   case 7:      //direct indirect long
        return direct_indir_longVw();
   case 9:      //immediate
        return SNESgetword(PC=+2,PB);
   case 0xd:    //absolute
        return absoluteVw();
   case 0xf:    //absolute
        return absolute_longVw();
   case 0x11:   //direct Indirect indexed
        return direct_indir_indexed_yVw();
   case 0x12:   //direct indirect
        return direct_indirVw();
   case 0x13:   //stack relative indirect indexed
        debug();
        break;
   case 0x15:   //direct indexed x
        return direct_indexed_xVw();
   case 0x17:   //direct indirect indexed long
        debug();
        break;
   case 0x19:   //absolute indexed (y)
        return absolute_YindexedVw();
   case 0x1D:
        return absolute_XindexedVw();
   case 0x1F:   //absolute indexed long
        return absolute_long_indexedVw();
   }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

inline unsigned long int family1a()
{
 unsigned char optest;
 unsigned long int tmp;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 1:      //direct indexed indirect
        return direct_indexed_indir();
   case 3:      //stack relative
        debug();
        break;
   case 5:      //direct
        return direct();
   case 7:      //direct indirect long
        return direct_indir_long();
   case 9:      //immediate
        tmp=PC;
        PC+=2;
        return tmp;
   case 0xd:    //absolute
        return absolute();
   case 0xf:    //absolute
        return absolute_long();
   case 0x11:   //direct Indirect indexed
        return direct_indir_indexed_y();
   case 0x12:   //direct indirect
        return direct_indir();
   case 0x13:   //stack relative indirect indexed
        debug();
        break;
   case 0x15:   //direct indexed x
        return direct_indexed_x();
   case 0x17:   //direct indirect indexed long
        debug();
        break;
   case 0x19:   //absolute indexed (y)
        return absolute_Yindexed();
   case 0x1D:
        return absolute_Xindexed();
   case 0x1F:   //absolute indexed long
        return absolute_long_indexed();
   }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

inline unsigned char family2b()
{
 unsigned char optest;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 2:
        return SNESgetbyte(PC++,PB);
   case 6:    //direct
        return directVb();
   case 0xE:  //Absolute
        return absoluteVb();
   case 0x16: //direct indexed (y)
        return direct_indexed_yVb();
   case 0x1E: //absolute indexed (y)
        return absolute_YindexedVb();
   }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

inline unsigned short int family2w()
{
 unsigned char optest;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 2:
        PC+=2;
        return SNESgetword(PC-2,PB);
   case 6:    //direct
        return directVw();
   case 0xE:  //Absolute
        return absoluteVw();
   case 0x16: //direct indexed (y)
        return direct_indexed_yVw();
   case 0x1E: //absolute indexed (y)
        return absolute_YindexedVw();
   }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

inline unsigned long int family2a()
{
 unsigned char optest;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 2:
        return SNESgetbyte(PC++,PB);
   case 6:    //direct
        return direct();
   case 0xE:  //Absolute
        return absolute();
   case 0x16: //direct indexed (y)
        return direct_indexed_y();
   case 0x1E: //absolute indexed (y)
        return absolute_Yindexed();
   }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

inline unsigned char family3b()
{
 unsigned char optest;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 0:    //immediate
        return SNESgetbyte(PC++,PB);
   case 4:    //direct
        return directVb();
   case 0xC:  //absolute
        return absoluteVb();
   case 0x14: //direct indexed x
        return direct_indexed_xVb();
   case 0x1C: //absolute indexed (x)
        return absolute_XindexedVb();
 }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

inline unsigned short int family3w()
{
 unsigned char optest;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 0:    //immediate
        PC+=2;
        return SNESgetword(PC-2,PB);
   case 4:    //direct
        return directVw();
   case 0xC:  //absolute
        return absoluteVw();
   case 0x14: //direct indexed x
        return direct_indexed_xVw();
   case 0x1C: //absolute indexed (X)
        return absolute_XindexedVw();
   }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

inline unsigned long int family3a()
{
 unsigned char optest;
 optest=SNESgetbyte(PC,PB)&0x1F;
 PC++;
 switch (optest) {
   case 0:    //immediate
        debug();
        break;
   case 4:    //direct
        return direct();
   case 0xC:  //absolute
        return absolute();
   case 0x14: //direct indexed x
        return direct_indexed_x();
   case 0x1C: //Absolute indexed (x)
        return absolute_Xindexed();
   }
 puts("I don't know what to do cap'n!");
 PC--;
 debug();
 return 0;
}

//Opcodes begin....

void _AND_()
{
 #ifdef debug1
 printf("and\n");
 #endif
 P.Z=0;
 if (P.M==1)
   {
    A=A&(family1b()&0xFF);
    if (A==0) P.Z=1;
   }
 else
   {
    A=A&(family1w()&0xFFFF);
    if (A==0) P.Z=1;
   }
 t+=2;
}

void sei()
{
 #ifdef debug1
 printf("sei\n");
 #endif
 P.I=1;
 t+=2;
 PC++;
}


void cld()
{
 #ifdef debug1
 printf("cld\n");
 #endif
 P.D=0;
 t+=2;
 PC++;
}

void clc()
{
 #ifdef debug1
 printf("clc\n");
 #endif
 P.C=0;
 t+=2;
 PC++;
}

void xce()
{
 int tmp;
 #ifdef debug1
 printf("xce\n");
 #endif
 tmp=P.E;
 P.E=P.C;
 P.C=tmp;
 t+=2;
 PC++;
}

void txa()
{
 #ifdef debug1
 printf("txa\n");
 #endif
 A=X;
 t+=2;
 PC++;
}

void tya()
{
 #ifdef debug1
 printf("tya\n");
 #endif
 A=Y;
 t+=2;
 PC++;
}

void tax()
{
 #ifdef debug1
 printf("tax   7%% GST 7%% PST  ;)\n");
 #endif
 X=A;
 t+=2;
 PC++;
}

void phk()
{
 //int location;
 //MemType typ;

 #ifdef debug1
 printf("phk\n");
 #endif
 pushb(PB);
 //SnesMap(S, 0, &typ, (long int*)&location);
 //SNESsetbyte(typ,location,PB);
 S++;
 t+=3;
 PC++;
}

void plb()
{
 //int location;
 //MemType typ;

 #ifdef debug1
 printf("plb\n");
 #endif
 DB=pullb();
 //S--;
 //DB=SNESgetbyte(S,0);
 t+=4;
 PC++;
}

void rep()            // Optimize!!!!  Bad Coding.
{
 int tmp;
 #ifdef debug1
 printf("rep #%x\n",SNESgetbyte(PC+1,PB));
 #endif
 PC++;
 tmp=SNESgetbyte(PC,PB);
 if ((tmp&0x80)==0x80) P.N=0;
 if ((tmp&0x40)==0x40) P.V=0;
 if ((tmp&0x20)==0x20) P.M=0;
 if ((tmp&0x10)==0x10) P.X=0;
 if ((tmp&0x08)==0x08) P.D=0;
 if ((tmp&0x04)==0x04) P.I=0;
 if ((tmp&0x02)==0x02) P.Z=0;
 if ((tmp&0x01)==0x01) P.C=0;
 t+=3;
 PC++;
}

void sep()            // Optimize!!!!  Bad Coding.  Same as rep
{
 int tmp;
 #ifdef debug1
 printf("sep #%x\n",SNESgetbyte(PC+1,PB));
 #endif
 PC++;
 tmp=SNESgetbyte(PC,PB);
 if ((tmp&0x80)==0x80) P.N=1;
 if ((tmp&0x40)==0x40) P.V=1;
 if ((tmp&0x20)==0x20) P.M=1;
 if ((tmp&0x10)==0x10) P.X=1;
 if ((tmp&0x08)==0x08) P.D=1;
 if ((tmp&0x04)==0x04) P.I=1;
 if ((tmp&0x02)==0x02) P.Z=1;
 if ((tmp&0x01)==0x01) P.C=1;
 t+=3;
 PC++;
}

void _LDA_()
{
 #ifdef debug1
 printf("lda\n");
 #endif
 if (P.M==1)
   A=family1b()&0xFF;
 else
   A=family1w()&0xFFFF;
 t+=2;
}


void _STA_()
{
 unsigned long int addr;
 int location;
 MemType typ;

 #ifdef debug1
 printf("sta ");
 #endif
 if (P.M==1)
   {
    addr=family1a();
    #ifdef debug1
    printf("%06x\n",addr);
    #endif
    SnesMap(addr&0xFFFF,addr>>16, &typ, (long int*)&location);
    SNESsetbyte(typ,location,(char)(A&0xff));
   }
  else
   {
    addr=family1a();
    #ifdef debug1
    printf("%06x\n",addr);
    #endif
    SnesMap(addr&0xFFFF,addr>>16, &typ, (long int*)&location);
    SNESsetword(typ,location,A);
   }
 t+=2;
}

void _LDX_()
{
 #ifdef debug1
 printf("ldx\n");
 #endif
 if (P.X==1)
   X=family2b()&0xFF;
 else
   X=family2w()&0xFFFF;
 t+=2;
}

void nop()      //heh  even easier... No brains here.  ;)
{
 #ifdef debug1
 printf("nop\n");
 #endif
 PC++;
 t+=2;
}

void inx()      // another easy one...
{
 #ifdef debug1
 printf("inx\n");
 #endif
 X++;
 if (P.X==1)
   X&=0xff;
 t+=2;
 PC++;
}

void iny()      // another easy one...
{
 #ifdef debug1
 printf("iny\n");
 #endif
 Y++;
 if (P.X==1)
   Y&=0xff;
 t+=2;
 PC++;
}

void _LDY_()
{
 #ifdef debug1
 printf("ldy\n");
 #endif
 if (P.X==1)
   Y=family3b()&0xFF;
 else
   Y=family3w()&0xFFFF;
 t+=2;
}

void _CPX_()
{
 #ifdef debug1
 printf("cpx\n");
 #endif
 P.Z=1;
 if (P.X==1)
   {
    if ((X&0xFF)==(family3b()&0xFF)) P.Z=0;
   }
 else
   {
    if (X==family3w()) P.Z=0;
   }
 t+=2;
}

void bne()
{
 #ifdef debug1
 printf("bne relative:%i\n",(signed char)SNESgetbyte(PC+1,PB));
 #endif
 if (P.Z!=0)
   PC=PC+(signed char)(SNESgetbyte(PC+1,PB));
 PC+=2;
 #ifdef debug1
 printf("P.Z==%i\n",P.Z);
 #endif
 t+=2;
}

void _STX_()
{
 unsigned long int addr;
 int location;
 MemType typ;

 #ifdef debug1
 printf("stx ");
 #endif
 if (P.X==1)
   {
    addr=family2a();
    #ifdef debug1
    printf("%06x\n",addr);
    #endif
    SnesMap(addr&0xFFFF,addr>>16, &typ, (long int*)&location);
    SNESsetbyte(typ,location,(char)(X&0xff));
   }
  else
   {
    addr=family2a();
    #ifdef debug1
    printf("%06x\n",addr);
    #endif
    SnesMap(addr&0xFFFF,addr>>16, &typ, (long int*)&location);
    SNESsetword(typ,location,X);
   }
 t+=2;
}

void stz_a()
{
  int location;
  MemType typ;
  #ifdef debug1
  puts("stz a ");
  #endif
  PC++;

  SnesMap(SNESgetword(PC,PB), DB, &typ, (long int*)&location);
  SNESsetbyte(typ,location,0);
  PC+=2;
  t+=4;
}

void stz_ax()
{
  int location;
  MemType typ;
  #ifdef debug1
  puts("stz a,y ");
  #endif
  PC++;

  SnesMap(SNESgetword(PC,PB)+X, DB, &typ, (long int*)&location);
  SNESsetbyte(typ,location,0);
  PC+=2;
  t+=4;
}

void xba()
{
 unsigned char tmp;
 #ifdef debug1
 printf("xba\n");
 #endif
 tmp=A&0xFF;
 A=(A>>8)&(tmp<<8);
 t+=3;
 PC++;
}

void jsr_a()
{
 //int location;
 //MemType typ;

 #ifdef debug1
 printf("jsr a\n");
 #endif
 PC+=3;
 pushb(PB);
 pushw(PC);
 PC=SNESgetword(PC-2,PB);
 t+=6;
}

void php()         //optimize
{
 unsigned char tmp;
 #ifdef debug1
 printf("php\n");     //Don't critize me.  It's processor independant.
 #endif
 tmp=P.C|(((int)P.Z)<<1)|(((int)P.I)<<2)|(((int)P.D)<<3)|(((int)P.X)<<4)|(((int)P.M)<<5)|(((int)P.V)<<6)|(((int)P.N)<<7);
 pushb(tmp);
 t+=3;
 PC++;
}

void plp()        //optimize
{
 unsigned char tmp;
 #ifdef debug1
 printf("plp\n");     //Don't critize me.  It's processor independant.
 #endif
 tmp=pullb();
 if (tmp&1) P.C=1;
 if (tmp&2) P.Z=1;
 if (tmp&4) P.I=1;
 if (tmp&8) P.D=1;
 if (tmp&16) P.X=1;
 if (tmp&32) P.M=1;
 if (tmp&64) P.V=1;
 if (tmp&128) P.N=1;
 t+=3;
 PC++;
}

void _STY_()
{
 unsigned long int addr;
 int location;
 MemType typ;

 #ifdef debug1
 printf("sty ");
 #endif
 if (P.X==1)
   {
    addr=family3a();
    #ifdef debug1
    printf("%06x\n",addr);
    #endif
    SnesMap(addr&0xFFFF,addr>>16, &typ, (long int*)&location);
    SNESsetbyte(typ,location,(char)(Y&0xff));
   }
  else
   {
    addr=family3a();
    #ifdef debug1
    printf("%06x\n",addr);
    #endif
    SnesMap(addr&0xFFFF,addr>>16, &typ, (long int*)&location);
    SNESsetword(typ,location,Y);
   }
 t+=2;
}

void beq()
{
 #ifdef debug1
 printf("beq relative:%i\n",(signed char)SNESgetbyte(PC+1,PB));
 #endif
 if (P.Z==0)
   PC=PC+(signed char)(SNESgetbyte(PC+1,PB));
 PC+=2;
 #ifdef debug1
 printf("P.Z==%i\n",P.Z);
 #endif
 t+=2;
}

void rts()
{
 #ifdef debug1
 printf("rts\n");
 #endif
 PC=pullw();
 PB=pullb();
 t+=6;
}

void bra()
{
 #ifdef debug1
 printf("bra\n");
 #endif
 PC=PC+2+(signed char)(SNESgetbyte(PC+1,PB));
 t+=2;
}

void _CPY_()
{
 #ifdef debug1
 printf("cpy\n");
 #endif
 P.Z=1;
 if (P.X==1)
   {
    if ((Y&0xFF)==(family3b()&0xFF)) P.Z=0;
   }
 else
   {
    if (Y==family3w()) P.Z=0;
   }
 t+=2;
}

void cli()
{
 #ifdef debug1
 printf("cli\n");
 #endif
 P.I=0;
 t+=2;
 PC++;
}

void _ADC_()
{
 long int tmpresult;
 #ifdef debug1
 printf("adc\n");
 #endif
 if (P.M==1)
   {
    tmpresult=(A&0xFF)+family1b();
    if (tmpresult>0xFF) P.C=1;
    A=tmpresult;
   }
 else
   {
    tmpresult=(A&0xFFFF)+family1w();
    if (tmpresult>0xFFFF) P.C=1;
    A=tmpresult;
   }
 t+=2;
}

void tay()
{
 #ifdef debug1
 printf("tay\n");
 #endif
 Y=A;
 t+=2;
 PC++;
}

void pha()
{
 //int location;
 //MemType typ;

 #ifdef debug1
 printf("pha\n");
 #endif
 if (P.M==1)
   pushb(A);
 else
   pushw(A);
 //SnesMap(S, 0, &typ, (long int*)&location);
 //SNESsetbyte(typ,location,PB);
 //S++;
 t+=3;
 PC++;
}


