/*****************************************************************************/
/*                                                                           */
/* Module:  Test                                                             */
/* Purpose: Tests the SOUND.C and TIASOUND.C modules.  Also, provides an     */
/*          example on their use. V1.0.                                      */
/* Author:  Ron Fries                                                        */
/* Date:    September 10, 1996                                               */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*                 License Information and Copyright Notice                  */
/*                 ========================================                  */
/*                                                                           */
/* TiaSound is Copyright(c) 1996 by Ron Fries                                */
/*                                                                           */
/* This library is free software; you can redistribute it and/or modify it   */
/* under the terms of version 2 of the GNU Library General Public License    */
/* as published by the Free Software Foundation.                             */
/*                                                                           */
/* This library is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library */
/* General Public License for more details.                                  */
/* To obtain a copy of the GNU Library General Public License, write to the  */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.   */
/*                                                                           */
/* Any permitted reproduction of these routines, in whole or in part, must   */
/* bear this legend.                                                         */
/*                                                                           */
/*****************************************************************************/

#include <conio.h>
#include <stdio.h>     
#include <ctype.h>
#include <stdlib.h>  

#include "sound.h"
#include "tiasound.h"

int main(int argc, char *argv[])
{
   char done = 0;
   char c;
   unsigned char AUDF[2] = {0,0},
                 AUDV[2] = {0,0},
                 AUDC[2] = {0,0};
   char chan;
   char x;
   unsigned int playback_freq = 15700;
   unsigned int sample_freq = 31400;
   unsigned int num_bufs = 2;
   unsigned int buffer_size = 400;

   printf ("\n");
   printf ("                        Tia Sound Generator Test Program\n");
   printf ("                        ================================\n");
   printf ("\n");
   printf ("Command line options:\n");
   printf ("\n");
   printf ("SNDTEST [-fxxxxx] [-pxxxxx] [-nx] [-sxxxxx]\n");
   printf ("\n");
   printf ("   -f Sets the sample frequency, default = 31400\n");
   printf ("   -p Sets the playback frequency, default = 22050\n");
   printf ("   -n Sets the number of playback buffers, default = 2\n");
   printf ("   -s Sets the size of each playback buffer, default = 1600\n");
   printf ("\n");
   printf ("All parameters are optional.\n");
   printf ("\n");
   printf ("Use keys 'a'-'g' and 'z'-'b' to adjust\n");
   printf ("the AUDC, AUDF and AUDV parameters.\n");
   printf ("\n");

   x = 1;
   while (x<argc)
   {
      if (argv[x][0] == '-')
      {
         switch (toupper(argv[x][1]))
         {
            case 'F':
               sample_freq = atoi (argv[x]+2);
               break;
            
            case 'P':
               playback_freq = atoi (argv[x]+2);
               break;
              
            case 'N':
               num_bufs = atoi (argv[x]+2);
               break;

            case 'S':
               buffer_size = atoi (argv[x]+2);
               break;
         }
      }
      x++;
   }


   printf ("Sample Frequency set to: %d\n",sample_freq);
   printf ("Playback Frequency set to: %d\n",playback_freq);
   printf ("Number of Buffers: %d\n",num_bufs);
   printf ("Buffer Size: %d\n",buffer_size);
   printf ("\n");

   OpenSB (playback_freq, num_bufs, buffer_size);

   Tia_sound_init (sample_freq, playback_freq);

   while (!done)
   {
      while (!kbhit())
      {
         Fillbuffer();
      }

      c = getch();

      chan = 0;      /* assume channel 0 */

      switch (toupper(c))
      {
         case 'Q':
           done = 1;
           break;

         case 'F':
           chan = 1;
         case 'A':
           (AUDC[chan] >= 0x0f) ? AUDC[chan]=0x00 : AUDC[chan]++;
           Update_tia_sound (0x15 + chan, AUDC[chan]);
           break;


         case 'G':
           chan = 1;
         case 'S':
           (AUDF[chan] >= 0x1f) ? AUDF[chan]=0x00 : AUDF[chan]++;
           Update_tia_sound (0x17 + chan, AUDF[chan]);
           break;

         case 'H':
           chan = 1;
         case 'D':
           (AUDV[chan] == 0x0f) ? AUDV[chan]=0x00 : AUDV[chan]++;
           Update_tia_sound (0x19 + chan, AUDV[chan]);
           break;

         case 'V':
           chan = 1;
         case 'Z':
           (AUDC[chan] == 0x00) ? AUDC[chan]=0x0f : AUDC[chan]--;
           Update_tia_sound (0x15 + chan, AUDC[chan]);
           break;

         case 'B':
           chan = 1;
         case 'X':
           (AUDF[chan] == 0x00) ? AUDF[chan]=0x1f : AUDF[chan]--;
           Update_tia_sound (0x17 + chan, AUDF[chan]);
           break;

         case 'N':
           chan = 1;
         case 'C':
           (AUDV[chan] == 0x00) ? AUDV[chan]=0x0f : AUDV[chan]--;
           Update_tia_sound (0x19 + chan, AUDV[chan]);
           break;

      }

      printf ("\t\tC0=%02hx, F0=%02hx, V0=%02hx\tC1=%02hx, F1=%02hx, V1=%02hx\r",
         AUDC[0],AUDF[0],AUDV[0],AUDC[1],AUDF[1],AUDV[1]);
   }
   
   CloseSB();
   
   return (0);
}
