/*******************************************************************************

File: TREPDOS.CPP

PURPOSE: Main program.  Includes simple cartridge reading, etc.  Console Program.

Copyright 1997 Chad Kitching.  Read the readme.txt for license info.

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <io.h>
#include <dos.h>
#include <malloc.h>
#include <mem.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "snesstru.h"
#include "memory~1.h"
#include "65c816.h"

char FileName[255];
SNESRomInfoStruct GameInfo;
long int fileheader;
HLtype ROMType;
bool LoROMovr, HiROMovr, override;

//Gets info from the cart file.  All the vital stats, hehe
void GetStats(char *File)
{
  SNESRomInfoStruct GI1, GI2;
  int i;
  unsigned int j,k;
  FILE *file_ptr;

  if ((file_ptr=fopen(File,"rb"))==NULL) {
    puts("Cannot open file for reading...");
    exit(255);
  }
  fseek(file_ptr,0,SEEK_END);
  //fileheader=512;
  if ((fileheader=ftell(file_ptr)&8191)!=512)
    puts ("Headerless image");
   else
    {
     printf("%i-byte header detected\n",fileheader);
    }
  printf("Size: %li\n",ftell(file_ptr));
  fseek(file_ptr,0x7FC0+fileheader,SEEK_SET);
  fread(&GI1,36,1,file_ptr);
  fseek(file_ptr,0xFFC0+fileheader,SEEK_SET);
  fread(&GI2,sizeof(SNESRomInfoStruct),1,file_ptr);
  fclose(file_ptr);

  ROMType=LoROM;

  j=GI1.Complement + GI1.Checksum;
  k=GI2.Complement + GI2.Checksum;
  if ((j == 65535u)&&(k!=65535u)) {
     puts("LowROM image!");
     GameInfo=GI1;
     ROMType=LoROM; }
  else if ((j!=65535u) && (k==65535u)) {
     puts("HiROM image!");
     GameInfo=GI2;
     ROMType=HiROM; }
  else if (!override) {
     puts("Invalid Image, Abort!");
     exit(253);
  }
  if (HiROMovr && override) {
     ROMType=HiROM;
     GameInfo=GI2;
     puts("forced to HiROM!");
     }
     else if (LoROMovr && override)
     {
      ROMType=LoROM;
      GameInfo=GI1;
      puts("forced to LoROM!");
     }

  printf("Game title:");
  fflush(stdout);  //damned buffered output. DJGPP needs this, or things get out of order
  for (i=0;i<21;i++)
   {
    putchar(GameInfo.ROM_Title[i]);
   }
  puts("");
}


void Exitting(void)
{
}

int main(int argc, char *argv[])
{
  int i;
  int paused;
  puts("TrepSNES 0.3 Beta for DOS-32bit.                               FREEWARE!!");
  puts("Copyright 1997 Chad Kitching.");
  if (argc<=1) {
    printf("\nSyntax : %s [filename] -flags\n", argv[0]);
    printf("  Valid flags: l - Force LoROM mode\n               h - Force HiROM mode\n");
    printf("               v - Version information\n");//               p - turn off pause before execution\n");
    exit(254);
  }
  strcpy(FileName,"");
  override=false;
  paused=0;
  for (i=1;i<argc;i++) {
    if (*argv[i]!='-') {
      strcpy(FileName,argv[i]);
    } else {
      switch (int(argv[i][1])) {
        case 'l': LoROMovr=true;
                  HiROMovr=false;
                  override=true;
                  break;
        case 'h': HiROMovr=true;
                  LoROMovr=false;
                  override=true;
                  break;
        case 'v': printf("EXE file generated on %s %s CST.\n",__DATE__,__TIME__);
                  puts("Project began on      Jan 13 1997 13:38:33 CST.");
                  InitEngine();
                  #if _M_IX86==200
                  puts("286 version");
                  #elif _M_IX86==300
                  puts("386 version");
                  #elif _M_IX86==400
                  puts("486 version");
                  #elif _M_IX86==500
                  puts("Pentium version");
                  #elif _M_IX86==600
                  puts("Pentium Pro version -- I shouldn't have released it like this?");
                  #endif
                  puts("End of Version info");
                  puts("");
                  puts("C, n.: A programming language that is sort of like Pascal except more like \nassembly except that it isn't much like either or anything else. It is either \nthe best language available to the art today, or it isn't.");
                  exit(0);
                  break;
        case 'p': paused=1;
                  break;
        default : printf("\nInvalid option %c", argv[i][1]);
                  exit(253);
      }
    }
  }
  printf("\nFilename is %s\n", FileName);
  GetStats(FileName);
  AllocateRAM();
  atexit(FreeBuffers);
  LoadImage(FileName);
  //LoadSRAM(FileName);
  InitEngine();
  puts("Press key to begin!");
  getche();
  BeginEmulation();
  //SaveSRAM();
}
